/*
 *  Copyright (C) 2010-2016. Mellanox Technologies, Ltd. ALL RIGHTS RESERVED.
 *
 *    Licensed under the Apache License, Version 2.0 (the "License"); you may
 *    not use this file except in compliance with the License. You may obtain
 *    a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 *    THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR
 *    CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 *    LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS
 *    FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
 *
 *    See the Apache Version 2.0 License for specific language governing
 *    permissions and limitations under the License.
 *
 */


#ifndef __SX_HOST_H__
#define __SX_HOST_H__

#include <sx/sxd/sxdev.h>
#include <sx/sxd/sxd_host.h>
#include <sx/sdk/sx_event_id.h>
#include <sx/sdk/sx_trap_id.h>
#include <sx/sdk/sx_acl.h>

/************************************************
 *  Type definitions
 ***********************************************/

/**
 * SX_TRAP_GROUP_MAX defines the maximal value for a trap group
 */
#define SX_TRAP_GROUP_MAX 63

/**
 * sx_fd_t structure is used to store SwitchX driver file descriptor.
 */
typedef struct sx_fd {
    int        fd; /* File descriptor for the SXD device */
    sxd_handle driver_handle; /**< handle descriptor for SXD device */
    boolean_t  valid;    /**< Indicates if the descriptor is valid */
} sx_fd_t;

typedef struct ku_l2_tunnel_params sx_l2_tunnel_params_t;

typedef enum sx_user_channel_type {
    SX_USER_CHANNEL_TYPE_FD = SX_KU_USER_CHANNEL_TYPE_FD,
    SX_USER_CHANNEL_TYPE_L3_NETDEV = SX_KU_USER_CHANNEL_TYPE_L3_NETDEV,
    SX_USER_CHANNEL_TYPE_LOG_PORT_NETDEV = SX_KU_USER_CHANNEL_TYPE_L2_NETDEV,
    SX_USER_CHANNEL_TYPE_PHY_PORT_NETDEV = SX_KU_USER_CHANNEL_TYPE_PHY_NETDEV,
    SX_USER_CHANNEL_TYPE_L2_TUNNEL = SX_KU_USER_CHANNEL_TYPE_L2_TUNNEL,
    /* Backport legacy support */
    SX_USER_CHANNEL_TYPE_NET = SX_KU_USER_CHANNEL_TYPE_L3_NETDEV
} sx_user_channel_type_t;

typedef union sx_user_channel_data {
    sx_fd_t               fd;
    sx_l2_tunnel_params_t l2_tunnel_params;
} sx_user_channel_data_t;

typedef struct sx_user_channel {
    sx_user_channel_type_t type;
    sx_user_channel_data_t channel;
} sx_user_channel_t;

typedef enum sx_truncate_mode {
    SX_TRUNCATE_MODE_DISABLE,
    SX_TRUNCATE_MODE_ENABLE
} sx_truncate_mode_t;

typedef enum sx_control_type {
    SX_CONTROL_TYPE_DEFAULT = 0,
    SX_CONTROL_TYPE_ENABLE,
    SX_CONTROL_TYPE_DISABLE,
} sx_control_type_t;

typedef enum sx_trap_id_user_defined_key_type {
    SX_TRAP_ID_USER_DEFINED_KEY_PACKET_TYPE_E = 0,
    SX_TRAP_ID_USER_DEFINED_KEY_PORT_RANGE_E,
} sx_trap_id_user_defined_key_type_t;

/**
 * sx_truncate_size_t is used to store the truncate size
 */
typedef uint16_t sx_truncate_size_t;

/**
 * sx_trap_group_t is used to note trap group
 */
typedef uint32_t sx_trap_group_t;

/**
 * sx_receive_info_t structure is used to store received
 * packet data
 */
typedef struct sx_host_ifc_receive_info {
    sx_trap_id_t     trap_id;    /**< trap ID */
    sx_event_info_t  event_info;    /**< event info */
    sx_port_log_id_t source_log_port;    /**< source logical port */
    boolean_t        is_lag; /**< Is the source logical port member of a lag */
    sx_port_log_id_t source_lag_port;    /**< source lag port when applicable */
    uint32_t         original_packet_size; /**<the original size of packet in bytes,
                                            *  if packet wasn't truncated packet_size=original_packet_size*/
} sx_receive_info_t;

/**
 * sx_trap_group_attributes_t structure is used to store the trap groups
 * attributes
 */
typedef struct sx_trap_group_attributes {
    sx_trap_priority_t prio;
    sx_truncate_mode_t truncate_mode;
    sx_truncate_size_t truncate_size;
    sx_control_type_t  control_type;
} sx_trap_group_attributes_t;

typedef enum sx_trap_id_user_defined_packet_type {
    SX_TRAP_ID_USER_DEFINED_PACKET_TYPE_UDP_E = 0,
    SX_TRAP_ID_USER_DEFINED_PACKET_TYPE_TCP_E,
    SX_TRAP_ID_USER_DEFINED_PACKET_TYPE_UDP_TCP_E,
} sx_trap_id_user_defined_packet_type_t;

typedef sx_acl_port_range_entry_t sx_host_ifc_port_range_t;

typedef union sx_trap_id_user_defined_key_data {
    sx_trap_id_user_defined_packet_type_t packet_type;
    sx_host_ifc_port_range_t              port_range;
} sx_trap_id_user_defined_key_data_t;

typedef struct sx_trap_id_user_defined_key {
    sx_trap_id_user_defined_key_type_t type;
    sx_trap_id_user_defined_key_data_t key;
} sx_trap_id_user_defined_key_t;

/**
 * sx_trap_id_user_defined_attributes_t structure is used to store the trap IDs
 * attributes
 */
typedef struct sx_trap_id_user_defined_attributes {
    sx_trap_id_user_defined_key_t key_list[RM_API_HOST_IFC_USER_DEFINE_KEY_MAX];
    uint32_t                      key_list_cnt;
} sx_trap_id_user_defined_attributes_t;

/**
 * host interface counter types
 */
typedef enum {
    HOST_IFC_COUNTER_TYPE_TRAP_GROUP_E,
    HOST_IFC_COUNTER_TYPE_TRAP_ID_E
} host_ifc_counter_type_e;

typedef enum {
    HOST_IFC_TRAP_TYPE_PACKET,
    HOST_IFC_TRAP_TYPE_EVENT
} host_ifc_trap_type_e;

/**
 * filter of host interface counters
 */
typedef struct sx_host_ifc_counters_filter {
    host_ifc_counter_type_e counter_type;
    union {
        struct {
            sx_trap_group_t trap_group_filter_list[SX_TRAP_GROUP_MAX];
            uint32_t        trap_group_filter_cnt;
        } trap_group;
        struct {
            sx_trap_id_t trap_id_filter_list[SX_TRAP_ID_MAX];
            uint32_t     trap_id_filter_cnt;
        } trap_id;
    } u_counter_type;
} sx_host_ifc_counters_filter_t;

/**
 * host interface global counters
 */
typedef struct sx_host_ifc_global_counters {
    uint64_t event_counter;
    uint64_t tocpu_packet;
    uint64_t tocpu_byte;
    uint64_t tocpu_buffer_drop;
    uint64_t fromcpu_data_packet;
    uint64_t fromcpu_data_byte;
    uint64_t fromcpu_control_packet;
    uint64_t fromcpu_control_byte;
} sx_host_ifc_global_counters_t;

/**
 * host interface trap group counters
 */
typedef struct sx_host_ifc_trap_group_counters {
    sx_trap_group_t       trap_group;
    uint64_t              event_counter;
    uint64_t              tocpu_packet;
    uint64_t              tocpu_byte;
    sx_policer_counters_t tocpu_drop_exceed_rate_packet;
} sx_host_ifc_trap_group_counters_t;

/**
 * host interface trap id counters
 */
typedef struct sx_host_ifc_trap_id_counters {
    sx_trap_id_t         trap_id;
    sx_trap_group_t      trap_group;
    host_ifc_trap_type_e trap_type;
    union {
        struct {
            uint64_t event_counter;
        } event;
        struct {
            uint64_t tocpu_packet;
            uint64_t tocpu_byte;
        } packet;
    } u_trap_type;
} sx_host_ifc_trap_id_counters_t;

/**
 * host interface counters main structure
 */
typedef struct sx_host_ifc_counters {
    sx_host_ifc_global_counters_t     global_counters;
    sx_host_ifc_trap_group_counters_t trap_group_counters[SX_TRAP_GROUP_MAX];
    uint32_t                          trap_group_counters_cnt;
    sx_host_ifc_trap_id_counters_t    trap_id_counters[SX_TRAP_ID_MAX];
    uint32_t                          trap_id_counters_cnt;
} sx_host_ifc_counters_t;

#endif /* __SX_HOST_H__ */
